/*
Copyright (c) 1992 Sandia National Laboratories

  Redistribution and use in source and binary forms are permitted
  provided that source distributions retain this entire copyright
  notice.  Neither the name of Sandia National Laboratories nor
  the name of the author may be used to endorse or promote products
  derived from this software without specific prior written permission.

Warranty:
  This software is provided "as is" and without any express or
  implied warranties, including, without limitation, the implied
  warranties of merchantability and fitness for a particular
  purpose.

Package title:
  Out-of-core LU factorization for nCUBE (Vertex)

General Description:
  This code computes the LU factorization of a matrix on an nCUBE
  parallel computer running the Vertex operating system
  The matrices L is not available explicitly, but rather as a 
  product of Gauss matrices and permutation matrices.

Author: 
  David E. Womble
  Sandia National Laboratories, Dept. 1422
  P.O. Box 5800
  Albuquerque, NM 87185-5800

  dewombl@cs.sandia.gov
  (505) 845-7471

Author's conditions of release:
  1. The author received acknowledgement in any publication or in 
     any distributions of code in which this code is used
  2. The author is notified of any problems encountered with the
     code

Notes:
  Notes on the design, behavior and use of the program are in the
  README file.
*/

#include <math.h>
#include <stdio.h>
#include <fcntl.h>


init_seg(seg, seg_num)
double *seg;
int seg_num;
{
#include "extern_global_vars.h"

  int k, l;          /* loop counters */
  int my_cols_this;  /* num of cols I have in a seg */
  double *tp;        /* temp ptr for stepping through the matrix entries */

  my_cols_this = (seg_num == nsegs_row-1) ? my_cols_last : my_cols_seg;

  for (l=0; l<my_cols_this; l++){
    tp = seg + l*(my_rows + 1);
    for (k=0; k<my_rows; k++){
      *tp++ = ( seg_num*ncols_seg + l*nprocs_row + my_first_col + 
                k*nprocs_col + my_first_row ) % nrows_matrix;
    }
  }
}

