/*
Copyright (c) 1992 Sandia National Laboratories

  Redistribution and use in source and binary forms are permitted
  provided that source distributions retain this entire copyright
  notice.  Neither the name of Sandia National Laboratories nor
  the name of the author may be used to endorse or promote products
  derived from this software without specific prior written permission.

Warranty:
  This software is provided "as is" and without any express or
  implied warranties, including, without limitation, the implied
  warranties of merchantability and fitness for a particular
  purpose.

Package title:
  Out-of-core LU factorization for nCUBE (Vertex)

General Description:
  This code computes the LU factorization of a matrix on an nCUBE
  parallel computer running the Vertex operating system
  The matrices L is not available explicitly, but rather as a 
  product of Gauss matrices and permutation matrices.

Author: 
  David E. Womble
  Sandia National Laboratories, Dept. 1422
  P.O. Box 5800
  Albuquerque, NM 87185-5800

  dewombl@cs.sandia.gov
  (505) 845-7471

Author's conditions of release:
  1. The author received acknowledgement in any publication or in 
     any distributions of code in which this code is used
  2. The author is notified of any problems encountered with the
     code

Notes:
  Notes on the design, behavior and use of the program are in the
  README file.
*/

#include	<stdio.h>


double seconds(start)
double start;			/* starting time */
{
   double time;			/* total seconds */
   long fulls;			/* full seconds */
   int micros = 1;		/* micro seconds */
   long n_time();

   fulls = n_time(&micros);
   time = fulls + micros/1.0e6 - start;

   return(time);
}

timing(secs, type, dest)
double secs;			/* number of ticks for this processor */
int type;			/* type of message to collect */
int dest;			/* processor who collects info */
{
   extern int me;		/* current processor number */
   extern int nprocs_cube;	/* total number of processors */
   double avt;			/* average number of ticks per processor */
   double maxt;			/* max ticks for processors */
   int imax;			/* processor where max ticks occurs */
   int from;			/* location a message comes from */
   int cflag;			/* message passing dummy parameter */
   int i;			/* loop counter */

   if (me != dest) {
      nwrite((char *) &secs, sizeof(double), dest, type, &cflag);
   }

   else {
      avt = maxt = secs;
      imax = dest;
      for (i=1; i<nprocs_cube; i++) {
	 from = -1;
	 nread((char *) &secs, sizeof(double), &from, &type, &cflag);
	 avt += secs;
	 if (secs > maxt) {
	    maxt = secs;
	    imax = from;
	 }
      }

      avt /= nprocs_cube;
      printf("%.4f (avg), %.4f (max on processor %d).\n",
	     avt, maxt, imax);

   }
}
